from __future__ import annotations

from core.constants import CellValue, checkCellValue
from .SetLayerValueCommand import SetLayerValueCommand
from ..Logic import Logic


class SetGroundValueCommand(SetLayerValueCommand):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("ground", value):
            return False
        coords = self._coords
        world = logic.world
        if not world.contains(coords):
            return False

        value = self._value
        groundValue = world.ground.getValue(coords)
        if value == groundValue:  # Value already set
            return False
        
        if value == CellValue.GROUND_SEA:  # Sea case
            impassableValue = world.impassable.getValue(coords)
            if impassableValue != CellValue.NONE:
                return False
            objectsValue = world.objects.getValue(coords)
            if objectsValue != CellValue.NONE:
                return False

        return True

    def execute(self, logic: Logic):
        coords = self._coords
        value = self._value
        ground = logic.world.ground
        ground.setValue(coords, value)
        ground.notifyCellChanged(coords)

        if self._fill:
            x, y = coords[0], coords[1]
            logic.addCommand(SetGroundValueCommand((x + 1, y), value, True))
            logic.addCommand(SetGroundValueCommand((x - 1, y), value, True))
            logic.addCommand(SetGroundValueCommand((x, y + 1), value, True))
            logic.addCommand(SetGroundValueCommand((x, y - 1), value, True))

