from __future__ import annotations

from core.constants import CellValue, checkCellValue
from .SetLayerValueCommand import SetLayerValueCommand
from ..Logic import Logic


class SetImpassableValueCommand(SetLayerValueCommand):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("impassable", value):
            return False
        coords = self._coords
        world = logic.world
        if not world.contains(coords):
            return False

        value = self._value
        impassableValue = world.impassable.getValue(coords)
        if value == CellValue.NONE:
            if impassableValue == CellValue.NONE:  # Value already removed
                return False
        else:
            if impassableValue != CellValue.NONE:  # Value already set
                return False
            if world.ground.getValue(coords) == CellValue.GROUND_SEA:
                return False
            if world.objects.getValue(coords) != CellValue.NONE:
                return False

        return True
        
    def execute(self, logic: Logic):
        coords = self._coords
        value = self._value
        world = logic.world
        world.impassable.setValue(coords, value)
        world.impassable.notifyCellChanged(coords)

        if self._fill:
            x, y = coords[0], coords[1]
            logic.addCommand(SetImpassableValueCommand((x + 1, y), value, True))
            logic.addCommand(SetImpassableValueCommand((x - 1, y), value, True))
            logic.addCommand(SetImpassableValueCommand((x, y + 1), value, True))
            logic.addCommand(SetImpassableValueCommand((x, y - 1), value, True))

