from abc import ABC, abstractmethod

from pygame import Surface

from ..Mouse import Mouse
from ..MouseWheel import MouseWheel
from ..theme.Theme import Theme


class GameMode(ABC):

    def __init__(self, theme: Theme):
        self.__theme = theme

    @property
    def theme(self):
        return self.__theme

    def dispose(self):
        pass

    @abstractmethod
    def processInput(self):
        raise NotImplementedError()

    @abstractmethod
    def update(self):
        raise NotImplementedError()

    @abstractmethod
    def render(self, surface: Surface):
        raise NotImplementedError()

    # Keyboard

    def keyDown(self, key: int):
        pass

    def keyUp(self, key: int):
        pass

    # Mouse handling

    def mouseEnter(self, mouse: Mouse):
        pass

    def mouseLeave(self):
        pass

    def mouseButtonDown(self, mouse: Mouse):
        pass

    def mouseButtonUp(self, mouse: Mouse):
        pass

    def mouseWheel(self, mouse: Mouse, wheel: MouseWheel):
        pass

    def mouseMove(self, mouse: Mouse):
        pass
